@@ -1,9 +1,13 @@
-use fnv::{FnvHashMap, FnvHashSet};
 use once_cell::sync::Lazy;
 use regex::Regex;
-use rustpython_ast::{Excepthandler, ExcepthandlerKind, Expr, ExprKind, Location, StmtKind};
+use rustc_hash::{FxHashMap, FxHashSet};
+use rustpython_ast::{
+    Arguments, Excepthandler, ExcepthandlerKind, Expr, ExprKind, Location, Stmt, StmtKind,
+};
+
+use crate::ast::types::Range;
+use crate::SourceCodeLocator;
 
-#[inline(always)]
 fn collect_call_path_inner<'a>(expr: &'a Expr, parts: &mut Vec<&'a str>) {
     match &expr.node {
         ExprKind::Call { func, .. } => {
@@ -21,7 +25,6 @@ fn collect_call_path_inner<'a>(expr: &'a Expr, parts: &mut Vec<&'a str>) {
 }
 
 /// Convert an `Expr` to its call path (like `List`, or `typing.List`).
-#[inline(always)]
 pub fn compose_call_path(expr: &Expr) -> Option<String> {
     let segments = collect_call_paths(expr);
     if segments.is_empty() {
@@ -32,7 +35,6 @@ pub fn compose_call_path(expr: &Expr) -> Option<String> {
 }
 
 /// Convert an `Expr` to its call path segments (like ["typing", "List"]).
-#[inline(always)]
 pub fn collect_call_paths(expr: &Expr) -> Vec<&str> {
     let mut segments = vec![];
     collect_call_path_inner(expr, &mut segments);
@@ -42,7 +44,7 @@ pub fn collect_call_paths(expr: &Expr) -> Vec<&str> {
 /// Rewrite any import aliases on a call path.
 pub fn dealias_call_path<'a>(
     call_path: Vec<&'a str>,
-    import_aliases: &FnvHashMap<&str, &'a str>,
+    import_aliases: &FxHashMap<&str, &'a str>,
 ) -> Vec<&'a str> {
     if let Some(head) = call_path.first() {
         if let Some(origin) = import_aliases.get(head) {
@@ -76,8 +78,8 @@ pub fn match_module_member(
     expr: &Expr,
     module: &str,
     member: &str,
-    from_imports: &FnvHashMap<&str, FnvHashSet<&str>>,
-    import_aliases: &FnvHashMap<&str, &str>,
+    from_imports: &FxHashMap<&str, FxHashSet<&str>>,
+    import_aliases: &FxHashMap<&str, &str>,
 ) -> bool {
     match_call_path(
         &dealias_call_path(collect_call_paths(expr), import_aliases),
@@ -94,7 +96,7 @@ pub fn match_call_path(
     call_path: &[&str],
     module: &str,
     member: &str,
-    from_imports: &FnvHashMap<&str, FnvHashSet<&str>>,
+    from_imports: &FxHashMap<&str, FxHashSet<&str>>,
 ) -> bool {
     // If we have no segments, we can't ever match.
     let num_segments = call_path.len();
@@ -118,10 +120,9 @@ pub fn match_call_path(
     // `Match`).
     if num_segments == 0 {
         module.is_empty()
-            || from_imports
-                .get(module)
-                .map(|imports| imports.contains(member) || imports.contains("*"))
-                .unwrap_or(false)
+            || from_imports.get(module).map_or(false, |imports| {
+                imports.contains(member) || imports.contains("*")
+            })
     } else {
         let components: Vec<&str> = module.split('.').collect();
 
@@ -145,8 +146,7 @@ pub fn match_call_path(
             let member = components[cut];
             if from_imports
                 .get(&module.as_str())
-                .map(|imports| imports.contains(member))
-                .unwrap_or(false)
+                .map_or(false, |imports| imports.contains(member))
             {
                 return true;
             }
@@ -215,6 +215,27 @@ pub fn extract_handler_names(handlers: &[Excepthandler]) -> Vec<Vec<&str>> {
     handler_names
 }
 
+/// Return the set of all bound argument names.
+pub fn collect_arg_names<'a>(arguments: &'a Arguments) -> FxHashSet<&'a str> {
+    let mut arg_names: FxHashSet<&'a str> = FxHashSet::default();
+    for arg in &arguments.posonlyargs {
+        arg_names.insert(arg.node.arg.as_str());
+    }
+    for arg in &arguments.args {
+        arg_names.insert(arg.node.arg.as_str());
+    }
+    if let Some(arg) = &arguments.vararg {
+        arg_names.insert(arg.node.arg.as_str());
+    }
+    for arg in &arguments.kwonlyargs {
+        arg_names.insert(arg.node.arg.as_str());
+    }
+    if let Some(arg) = &arguments.kwarg {
+        arg_names.insert(arg.node.arg.as_str());
+    }
+    arg_names
+}
+
 /// Returns `true` if a call is an argumented `super` invocation.
 pub fn is_super_call_with_arguments(func: &Expr, args: &[Expr]) -> bool {
     // Check: is this a `super` call?
@@ -250,7 +271,7 @@ pub fn to_module_and_member(target: &str) -> (&str, &str) {
 
 /// Convert a location within a file (relative to `base`) to an absolute
 /// position.
-pub fn to_absolute(relative: &Location, base: &Location) -> Location {
+pub fn to_absolute(relative: Location, base: Location) -> Location {
     if relative.row() == 1 {
         Location::new(
             relative.row() + base.row() - 1,
@@ -261,10 +282,38 @@ pub fn to_absolute(relative: &Location, base: &Location) -> Location {
     }
 }
 
+/// Return `true` if a `Stmt` has leading content.
+pub fn match_leading_content(stmt: &Stmt, locator: &SourceCodeLocator) -> bool {
+    let range = Range {
+        location: Location::new(stmt.location.row(), 0),
+        end_location: stmt.location,
+    };
+    let prefix = locator.slice_source_code_range(&range);
+    prefix.chars().any(|char| !char.is_whitespace())
+}
+
+/// Return `true` if a `Stmt` has trailing content.
+pub fn match_trailing_content(stmt: &Stmt, locator: &SourceCodeLocator) -> bool {
+    let range = Range {
+        location: stmt.end_location.unwrap(),
+        end_location: Location::new(stmt.end_location.unwrap().row() + 1, 0),
+    };
+    let suffix = locator.slice_source_code_range(&range);
+    for char in suffix.chars() {
+        if char == '#' {
+            return false;
+        }
+        if !char.is_whitespace() {
+            return true;
+        }
+    }
+    false
+}
+
 #[cfg(test)]
 mod tests {
     use anyhow::Result;
-    use fnv::{FnvHashMap, FnvHashSet};
+    use rustc_hash::{FxHashMap, FxHashSet};
     use rustpython_parser::parser;
 
     use crate::ast::helpers::match_module_member;
@@ -276,8 +325,8 @@ mod tests {
             &expr,
             "",
             "list",
-            &FnvHashMap::default(),
-            &FnvHashMap::default(),
+            &FxHashMap::default(),
+            &FxHashMap::default(),
         ));
         Ok(())
     }
@@ -289,8 +338,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::default(),
-            &FnvHashMap::default(),
+            &FxHashMap::default(),
+            &FxHashMap::default(),
         ));
         Ok(())
     }
@@ -302,16 +351,16 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::default(),
-            &FnvHashMap::default(),
+            &FxHashMap::default(),
+            &FxHashMap::default(),
         ));
         let expr = parser::parse_expression("re.Match", "<filename>")?;
         assert!(!match_module_member(
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::default(),
-            &FnvHashMap::default(),
+            &FxHashMap::default(),
+            &FxHashMap::default(),
         ));
         Ok(())
     }
@@ -323,8 +372,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::from_iter([("typing.re", FnvHashSet::from_iter(["*"]))]),
-            &FnvHashMap::default()
+            &FxHashMap::from_iter([("typing.re", FxHashSet::from_iter(["*"]))]),
+            &FxHashMap::default()
         ));
         Ok(())
     }
@@ -336,8 +385,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::from_iter([("typing.re", FnvHashSet::from_iter(["Match"]))]),
-            &FnvHashMap::default()
+            &FxHashMap::from_iter([("typing.re", FxHashSet::from_iter(["Match"]))]),
+            &FxHashMap::default()
         ));
         Ok(())
     }
@@ -349,8 +398,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::from_iter([("typing", FnvHashSet::from_iter(["re"]))]),
-            &FnvHashMap::default()
+            &FxHashMap::from_iter([("typing", FxHashSet::from_iter(["re"]))]),
+            &FxHashMap::default()
         ));
 
         let expr = parser::parse_expression("match.Match", "<filename>")?;
@@ -358,8 +407,8 @@ mod tests {
             &expr,
             "typing.re.match",
             "Match",
-            &FnvHashMap::from_iter([("typing.re", FnvHashSet::from_iter(["match"]))]),
-            &FnvHashMap::default()
+            &FxHashMap::from_iter([("typing.re", FxHashSet::from_iter(["match"]))]),
+            &FxHashMap::default()
         ));
 
         let expr = parser::parse_expression("re.match.Match", "<filename>")?;
@@ -367,8 +416,8 @@ mod tests {
             &expr,
             "typing.re.match",
             "Match",
-            &FnvHashMap::from_iter([("typing", FnvHashSet::from_iter(["re"]))]),
-            &FnvHashMap::default()
+            &FxHashMap::from_iter([("typing", FxHashSet::from_iter(["re"]))]),
+            &FxHashMap::default()
         ));
         Ok(())
     }
@@ -380,8 +429,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::from_iter([("typing.re", FnvHashSet::from_iter(["Match"]))]),
-            &FnvHashMap::from_iter([("IMatch", "Match")]),
+            &FxHashMap::from_iter([("typing.re", FxHashSet::from_iter(["Match"]))]),
+            &FxHashMap::from_iter([("IMatch", "Match")]),
         ));
         Ok(())
     }
@@ -393,8 +442,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::default(),
-            &FnvHashMap::from_iter([("t", "typing.re")]),
+            &FxHashMap::default(),
+            &FxHashMap::from_iter([("t", "typing.re")]),
         ));
         Ok(())
     }
@@ -406,8 +455,8 @@ mod tests {
             &expr,
             "typing.re",
             "Match",
-            &FnvHashMap::default(),
-            &FnvHashMap::from_iter([("t", "typing")]),
+            &FxHashMap::default(),
+            &FxHashMap::from_iter([("t", "typing")]),
         ));
         Ok(())
     }
