@@ -5,6 +5,19 @@ use crate::ast::types::Range;
 use crate::check_ast::Checker;
 use crate::checks::{Check, CheckKind};
 
+const FUNC_NAME_ALLOWLIST: &[&str] = &["get", "setdefault", "pop", "fromkeys"];
+
+/// Returns `true` if an argument is allowed to use a boolean trap. To return
+/// `true`, the function name must be explicitly allowed, and the argument must
+/// be either the first or second argument in the call.
+fn allow_boolean_trap(func: &Expr) -> bool {
+    if let ExprKind::Attribute { attr, .. } = &func.node {
+        FUNC_NAME_ALLOWLIST.contains(&attr.as_ref())
+    } else {
+        false
+    }
+}
+
 fn is_boolean_arg(arg: &Expr) -> bool {
     matches!(
         &arg.node,
@@ -51,7 +64,7 @@ pub fn check_boolean_default_value_in_function_definition(
     checker: &mut Checker,
     arguments: &Arguments,
 ) {
-    for arg in arguments.defaults.iter() {
+    for arg in &arguments.defaults {
         add_if_boolean(
             checker,
             arg,
@@ -60,8 +73,15 @@ pub fn check_boolean_default_value_in_function_definition(
     }
 }
 
-pub fn check_boolean_positional_value_in_function_call(checker: &mut Checker, args: &[Expr]) {
-    for arg in args {
+pub fn check_boolean_positional_value_in_function_call(
+    checker: &mut Checker,
+    args: &[Expr],
+    func: &Expr,
+) {
+    for (index, arg) in args.iter().enumerate() {
+        if index < 2 && allow_boolean_trap(func) {
+            continue;
+        }
         add_if_boolean(
             checker,
             arg,
