@@ -1,15 +1,26 @@
-#![allow(clippy::collapsible_if, clippy::collapsible_else_if)]
+#![allow(
+    clippy::collapsible_else_if,
+    clippy::collapsible_if,
+    clippy::implicit_hasher,
+    clippy::match_same_arms,
+    clippy::missing_errors_doc,
+    clippy::missing_panics_doc,
+    clippy::module_name_repetitions,
+    clippy::must_use_candidate,
+    clippy::similar_names,
+    clippy::too_many_lines
+)]
 
 use std::path::Path;
 
 use anyhow::Result;
 use log::debug;
+use rustpython_helpers::tokenize;
 use rustpython_parser::lexer::LexResult;
 use settings::{pyproject, Settings};
 
-use crate::autofix::fixer::Mode;
 use crate::checks::Check;
-use crate::linter::{check_path, tokenize};
+use crate::linter::check_path;
 use crate::settings::configuration::Configuration;
 use crate::source_code_locator::SourceCodeLocator;
 
@@ -24,9 +35,11 @@ pub mod checks;
 pub mod checks_gen;
 pub mod cli;
 pub mod code_gen;
+pub mod commands;
 mod cst;
 mod directives;
 mod docstrings;
+mod eradicate;
 mod flake8_2020;
 pub mod flake8_annotations;
 pub mod flake8_bandit;
@@ -35,6 +48,7 @@ pub mod flake8_boolean_trap;
 pub mod flake8_bugbear;
 mod flake8_builtins;
 mod flake8_comprehensions;
+mod flake8_debugger;
 mod flake8_print;
 pub mod flake8_quotes;
 pub mod flake8_tidy_imports;
@@ -51,13 +65,17 @@ pub mod printer;
 mod pycodestyle;
 mod pydocstyle;
 mod pyflakes;
+mod pygrep_hooks;
+mod pylint;
 mod python;
 mod pyupgrade;
 mod rules;
+mod rustpython_helpers;
 pub mod settings;
 pub mod source_code_locator;
 #[cfg(feature = "update-informer")]
 pub mod updates;
+mod vendored;
 pub mod visibility;
 
 /// Run Ruff over Python source code directly.
@@ -74,10 +92,10 @@ pub fn check(path: &Path, contents: &str, autofix: bool) -> Result<Vec<Check>> {
         None => debug!("Unable to find pyproject.toml; using default settings..."),
     };
 
-    let settings = Settings::from_configuration(Configuration::from_pyproject(
-        pyproject.as_ref(),
+    let settings = Settings::from_configuration(
+        Configuration::from_pyproject(pyproject.as_ref(), project_root.as_ref())?,
         project_root.as_ref(),
-    )?);
+    )?;
 
     // Tokenize once.
     let tokens: Vec<LexResult> = tokenize(contents);
@@ -89,7 +107,7 @@ pub fn check(path: &Path, contents: &str, autofix: bool) -> Result<Vec<Check>> {
     let directives = directives::extract_directives(
         &tokens,
         &locator,
-        &directives::Flags::from_settings(&settings),
+        directives::Flags::from_settings(&settings),
     );
 
     // Generate checks.
@@ -100,7 +118,8 @@ pub fn check(path: &Path, contents: &str, autofix: bool) -> Result<Vec<Check>> {
         &locator,
         &directives,
         &settings,
-        &if autofix { Mode::Generate } else { Mode::None },
+        autofix,
+        false,
     )?;
 
     Ok(checks)
