@@ -3,6 +3,7 @@ use std::fs;
 use std::path::Path;
 
 use anyhow::Result;
+use itertools::Itertools;
 use nohash_hasher::IntMap;
 use once_cell::sync::Lazy;
 use regex::Regex;
@@ -10,32 +11,40 @@ use regex::Regex;
 use crate::checks::{Check, CheckCode};
 
 static NO_QA_REGEX: Lazy<Regex> = Lazy::new(|| {
-    Regex::new(r"(?P<noqa>\s*(?i:# noqa)(?::\s?(?P<codes>([A-Z]+[0-9]+(?:[,\s]+)?)+))?)")
-        .expect("Invalid regex")
+    Regex::new(
+        r"(?P<spaces>\s*)(?P<noqa>(?i:# noqa)(?::\s?(?P<codes>([A-Z]+[0-9]+(?:[,\s]+)?)+))?)",
+    )
+    .unwrap()
 });
-static SPLIT_COMMA_REGEX: Lazy<Regex> = Lazy::new(|| Regex::new(r"[,\s]").expect("Invalid regex"));
+static SPLIT_COMMA_REGEX: Lazy<Regex> = Lazy::new(|| Regex::new(r"[,\s]").unwrap());
 
 #[derive(Debug)]
 pub enum Directive<'a> {
     None,
-    All(usize, usize),
-    Codes(usize, usize, Vec<&'a str>),
+    All(usize, usize, usize),
+    Codes(usize, usize, usize, Vec<&'a str>),
 }
 
 pub fn extract_noqa_directive(line: &str) -> Directive {
     match NO_QA_REGEX.captures(line) {
-        Some(caps) => match caps.name("noqa") {
-            Some(noqa) => match caps.name("codes") {
-                Some(codes) => Directive::Codes(
-                    noqa.start(),
-                    noqa.end(),
-                    SPLIT_COMMA_REGEX
-                        .split(codes.as_str())
-                        .map(|code| code.trim())
-                        .filter(|code| !code.is_empty())
-                        .collect(),
-                ),
-                None => Directive::All(noqa.start(), noqa.end()),
+        Some(caps) => match caps.name("spaces") {
+            Some(spaces) => match caps.name("noqa") {
+                Some(noqa) => match caps.name("codes") {
+                    Some(codes) => Directive::Codes(
+                        spaces.as_str().chars().count(),
+                        noqa.start(),
+                        noqa.end(),
+                        SPLIT_COMMA_REGEX
+                            .split(codes.as_str())
+                            .map(str::trim)
+                            .filter(|code| !code.is_empty())
+                            .collect(),
+                    ),
+                    None => {
+                        Directive::All(spaces.as_str().chars().count(), noqa.start(), noqa.end())
+                    }
+                },
+                None => Directive::None,
             },
             None => Directive::None,
         },
@@ -44,12 +53,13 @@ pub fn extract_noqa_directive(line: &str) -> Directive {
 }
 
 pub fn add_noqa(
+    path: &Path,
     checks: &[Check],
     contents: &str,
     noqa_line_for: &IntMap<usize, usize>,
-    path: &Path,
+    external: &BTreeSet<String>,
 ) -> Result<usize> {
-    let (count, output) = add_noqa_inner(checks, contents, noqa_line_for)?;
+    let (count, output) = add_noqa_inner(checks, contents, noqa_line_for, external);
     fs::write(path, output)?;
     Ok(count)
 }
@@ -58,7 +68,8 @@ fn add_noqa_inner(
     checks: &[Check],
     contents: &str,
     noqa_line_for: &IntMap<usize, usize>,
-) -> Result<(usize, String)> {
+    external: &BTreeSet<String>,
+) -> (usize, String) {
     let lines: Vec<&str> = contents.lines().collect();
     let mut matches_by_line: BTreeMap<usize, BTreeSet<&CheckCode>> = BTreeMap::new();
     for lineno in 0..lines.len() {
@@ -81,7 +92,7 @@ fn add_noqa_inner(
     }
 
     let mut count: usize = 0;
-    let mut output = "".to_string();
+    let mut output = String::new();
     for (lineno, line) in lines.iter().enumerate() {
         match matches_by_line.get(&lineno) {
             None => {
@@ -91,26 +102,76 @@ fn add_noqa_inner(
             Some(codes) => {
                 match extract_noqa_directive(line) {
                     Directive::None => {
-                        output.push_str(line);
+                        // Add existing content.
+                        output.push_str(line.trim_end());
+
+                        // Add `noqa` directive.
+                        output.push_str("  # noqa: ");
+
+                        // Add codes.
+                        let codes: Vec<&str> = codes.iter().map(AsRef::as_ref).collect();
+                        let suffix = codes.join(", ");
+                        output.push_str(&suffix);
+                        output.push('\n');
+                        count += 1;
+                    }
+                    Directive::All(_, start, _) => {
+                        // Add existing content.
+                        output.push_str(line[..start].trim_end());
+
+                        // Add `noqa` directive.
+                        output.push_str("  # noqa: ");
+
+                        // Add codes.
+                        let codes: Vec<&str> =
+                            codes.iter().map(AsRef::as_ref).sorted_unstable().collect();
+                        let suffix = codes.join(", ");
+                        output.push_str(&suffix);
+                        output.push('\n');
+                        count += 1;
+                    }
+                    Directive::Codes(_, start, _, existing) => {
+                        // Reconstruct the line based on the preserved check codes.
+                        // This enables us to tally the number of edits.
+                        let mut formatted = String::new();
+
+                        // Add existing content.
+                        formatted.push_str(line[..start].trim_end());
+
+                        // Add `noqa` directive.
+                        formatted.push_str("  # noqa: ");
+
+                        // Add codes.
+                        let codes: Vec<&str> = codes
+                            .iter()
+                            .map(AsRef::as_ref)
+                            .chain(existing.into_iter().filter(|code| external.contains(*code)))
+                            .sorted_unstable()
+                            .collect();
+                        let suffix = codes.join(", ");
+                        formatted.push_str(&suffix);
+
+                        output.push_str(&formatted);
+                        output.push('\n');
+
+                        // Only count if the new line is an actual edit.
+                        if &formatted != line {
+                            count += 1;
+                        }
                     }
-                    Directive::All(start, _) => output.push_str(&line[..start]),
-                    Directive::Codes(start, ..) => output.push_str(&line[..start]),
                 };
-                let codes: Vec<&str> = codes.iter().map(|code| code.as_ref()).collect();
-                output.push_str("  # noqa: ");
-                output.push_str(&codes.join(", "));
-                output.push('\n');
-                count += 1;
             }
         }
     }
 
-    Ok((count, output))
+    (count, output)
 }
 
 #[cfg(test)]
 mod tests {
-    use anyhow::Result;
+    use std::collections::BTreeSet;
+
+    use nohash_hasher::IntMap;
     use rustpython_parser::ast::Location;
 
     use crate::ast::types::Range;
@@ -132,11 +193,12 @@ mod tests {
     }
 
     #[test]
-    fn modification() -> Result<()> {
+    fn modification() {
         let checks = vec![];
         let contents = "x = 1";
-        let noqa_line_for = Default::default();
-        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for)?;
+        let noqa_line_for = IntMap::default();
+        let external = BTreeSet::default();
+        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for, &external);
         assert_eq!(count, 0);
         assert_eq!(output.trim(), contents.trim());
 
@@ -148,8 +210,9 @@ mod tests {
             },
         )];
         let contents = "x = 1";
-        let noqa_line_for = Default::default();
-        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for)?;
+        let noqa_line_for = IntMap::default();
+        let external = BTreeSet::default();
+        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for, &external);
         assert_eq!(count, 1);
         assert_eq!(output.trim(), "x = 1  # noqa: F841".trim());
 
@@ -170,8 +233,9 @@ mod tests {
             ),
         ];
         let contents = "x = 1  # noqa: E741";
-        let noqa_line_for = Default::default();
-        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for)?;
+        let noqa_line_for = IntMap::default();
+        let external = BTreeSet::default();
+        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for, &external);
         assert_eq!(count, 1);
         assert_eq!(output.trim(), "x = 1  # noqa: E741, F841".trim());
 
@@ -192,11 +256,10 @@ mod tests {
             ),
         ];
         let contents = "x = 1  # noqa";
-        let noqa_line_for = Default::default();
-        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for)?;
+        let noqa_line_for = IntMap::default();
+        let external = BTreeSet::default();
+        let (count, output) = add_noqa_inner(&checks, contents, &noqa_line_for, &external);
         assert_eq!(count, 1);
         assert_eq!(output.trim(), "x = 1  # noqa: E741, F841".trim());
-
-        Ok(())
     }
 }
