@@ -1,4 +1,4 @@
-use fnv::{FnvHashMap, FnvHashSet};
+use rustc_hash::{FxHashMap, FxHashSet};
 use rustpython_ast::{Arguments, Expr, ExprKind, Stmt};
 
 use crate::ast::types::{Range, Scope, ScopeKind};
@@ -11,13 +11,7 @@ use crate::python::string::{self};
 /// N801
 pub fn invalid_class_name(class_def: &Stmt, name: &str) -> Option<Check> {
     let stripped = name.strip_prefix('_').unwrap_or(name);
-    if !stripped
-        .chars()
-        .next()
-        .map(|c| c.is_uppercase())
-        .unwrap_or(false)
-        || stripped.contains('_')
-    {
+    if !stripped.chars().next().map_or(false, char::is_uppercase) || stripped.contains('_') {
         return Some(Check::new(
             CheckKind::InvalidClassName(name.to_string()),
             Range::from_located(class_def),
@@ -59,8 +53,8 @@ pub fn invalid_first_argument_name_for_class_method(
     name: &str,
     decorator_list: &[Expr],
     args: &Arguments,
-    from_imports: &FnvHashMap<&str, FnvHashSet<&str>>,
-    import_aliases: &FnvHashMap<&str, &str>,
+    from_imports: &FxHashMap<&str, FxHashSet<&str>>,
+    import_aliases: &FxHashMap<&str, &str>,
     settings: &Settings,
 ) -> Option<Check> {
     if matches!(
@@ -74,7 +68,14 @@ pub fn invalid_first_argument_name_for_class_method(
         ),
         FunctionType::ClassMethod
     ) {
-        if let Some(arg) = args.args.first() {
+        if let Some(arg) = args.posonlyargs.first() {
+            if arg.node.arg != "cls" {
+                return Some(Check::new(
+                    CheckKind::InvalidFirstArgumentNameForClassMethod,
+                    Range::from_located(arg),
+                ));
+            }
+        } else if let Some(arg) = args.args.first() {
             if arg.node.arg != "cls" {
                 return Some(Check::new(
                     CheckKind::InvalidFirstArgumentNameForClassMethod,
@@ -92,8 +93,8 @@ pub fn invalid_first_argument_name_for_method(
     name: &str,
     decorator_list: &[Expr],
     args: &Arguments,
-    from_imports: &FnvHashMap<&str, FnvHashSet<&str>>,
-    import_aliases: &FnvHashMap<&str, &str>,
+    from_imports: &FxHashMap<&str, FxHashSet<&str>>,
+    import_aliases: &FxHashMap<&str, &str>,
     settings: &Settings,
 ) -> Option<Check> {
     if matches!(
@@ -125,6 +126,11 @@ pub fn dunder_function_name(scope: &Scope, stmt: &Stmt, name: &str) -> Option<Ch
         return None;
     }
     if name.starts_with("__") && name.ends_with("__") {
+        // Allowed under PEP 562 (https://peps.python.org/pep-0562/).
+        if matches!(scope.kind, ScopeKind::Module) && (name == "__getattr__" || name == "__dir__") {
+            return None;
+        }
+
         return Some(Check::new(
             CheckKind::DunderFunctionName,
             Range::from_located(stmt),
