@@ -1,8 +1,13 @@
-use fnv::FnvHashMap;
+use anyhow::Result;
 use itertools::izip;
-use rustpython_parser::ast::{Cmpop, Constant, Expr, ExprKind, Unaryop};
+use log::error;
+use rustc_hash::FxHashMap;
+use rustpython_ast::{Arguments, Location, StmtKind};
+use rustpython_parser::ast::{Cmpop, Constant, Expr, ExprKind, Stmt, Unaryop};
 
+use crate::ast::helpers::{match_leading_content, match_trailing_content};
 use crate::ast::types::Range;
+use crate::ast::whitespace::leading_space;
 use crate::autofix::Fix;
 use crate::check_ast::Checker;
 use crate::checks::{Check, CheckKind, RejectedCmpop};
@@ -10,8 +15,8 @@ use crate::code_gen::SourceGenerator;
 
 fn compare(left: &Expr, ops: &[Cmpop], comparators: &[Expr]) -> Option<String> {
     let cmp = Expr::new(
-        Default::default(),
-        Default::default(),
+        Location::default(),
+        Location::default(),
         ExprKind::Compare {
             left: Box::new(left.clone()),
             ops: ops.to_vec(),
@@ -19,10 +24,9 @@ fn compare(left: &Expr, ops: &[Cmpop], comparators: &[Expr]) -> Option<String> {
         },
     );
     let mut generator = SourceGenerator::new();
-    if let Ok(()) = generator.unparse_expr(&cmp, 0) {
-        if let Ok(content) = generator.generate() {
-            return Some(content);
-        }
+    generator.unparse_expr(&cmp, 0);
+    if let Ok(content) = generator.generate() {
+        return Some(content);
     }
     None
 }
@@ -41,7 +45,7 @@ pub fn literal_comparisons(
     // through the list of operators, we apply "dummy" fixes for each error,
     // then replace the entire expression at the end with one "real" fix, to
     // avoid conflicts.
-    let mut bad_ops: FnvHashMap<usize, Cmpop> = FnvHashMap::default();
+    let mut bad_ops: FxHashMap<usize, Cmpop> = FxHashMap::default();
     let mut checks: Vec<Check> = vec![];
 
     let op = ops.first().unwrap();
@@ -260,3 +264,69 @@ pub fn not_tests(
         }
     }
 }
+
+fn function(name: &str, args: &Arguments, body: &Expr) -> Result<String> {
+    let body = Stmt::new(
+        Location::default(),
+        Location::default(),
+        StmtKind::Return {
+            value: Some(Box::new(body.clone())),
+        },
+    );
+    let func = Stmt::new(
+        Location::default(),
+        Location::default(),
+        StmtKind::FunctionDef {
+            name: name.to_string(),
+            args: Box::new(args.clone()),
+            body: vec![body],
+            decorator_list: vec![],
+            returns: None,
+            type_comment: None,
+        },
+    );
+    let mut generator = SourceGenerator::new();
+    generator.unparse_stmt(&func);
+    Ok(generator.generate()?)
+}
+
+/// E731
+pub fn do_not_assign_lambda(checker: &mut Checker, target: &Expr, value: &Expr, stmt: &Stmt) {
+    if let ExprKind::Name { id, .. } = &target.node {
+        if let ExprKind::Lambda { args, body } = &value.node {
+            let mut check = Check::new(CheckKind::DoNotAssignLambda, Range::from_located(stmt));
+            if checker.patch(check.kind.code()) {
+                if !match_leading_content(stmt, checker.locator)
+                    && !match_trailing_content(stmt, checker.locator)
+                {
+                    match function(id, args, body) {
+                        Ok(content) => {
+                            let indentation =
+                                &leading_space(&checker.locator.slice_source_code_range(&Range {
+                                    location: Location::new(stmt.location.row(), 0),
+                                    end_location: Location::new(stmt.location.row() + 1, 0),
+                                }));
+                            let mut indented = String::new();
+                            for (idx, line) in content.lines().enumerate() {
+                                if idx == 0 {
+                                    indented.push_str(line);
+                                } else {
+                                    indented.push('\n');
+                                    indented.push_str(indentation);
+                                    indented.push_str(line);
+                                }
+                            }
+                            check.amend(Fix::replacement(
+                                indented,
+                                stmt.location,
+                                stmt.end_location.unwrap(),
+                            ));
+                        }
+                        Err(e) => error!("Failed to generate fix: {}", e),
+                    }
+                }
+            }
+            checker.add_check(check);
+        }
+    }
+}
