@@ -1,20 +1,22 @@
 use std::collections::BTreeSet;
 
-use fnv::FnvHashSet;
 use itertools::Itertools;
 use once_cell::sync::Lazy;
 use regex::Regex;
-use rustpython_ast::{Arg, Constant, ExprKind, Location, StmtKind};
+use rustc_hash::FxHashSet;
+use rustpython_ast::{Constant, ExprKind, Location, StmtKind};
 
 use crate::ast::types::Range;
+use crate::ast::whitespace;
+use crate::ast::whitespace::LinesWithTrailingNewline;
 use crate::autofix::Fix;
 use crate::check_ast::Checker;
 use crate::checks::{Check, CheckCode, CheckKind};
+use crate::docstrings::constants;
 use crate::docstrings::definition::{Definition, DefinitionKind};
-use crate::docstrings::helpers;
 use crate::docstrings::sections::{section_contexts, SectionContext};
 use crate::docstrings::styles::SectionStyle;
-use crate::visibility::{is_init, is_magic, is_overload, is_staticmethod, Visibility};
+use crate::visibility::{is_init, is_magic, is_overload, is_override, is_staticmethod, Visibility};
 
 /// D100, D101, D102, D103, D104, D105, D106, D107
 pub fn not_missing(
@@ -87,7 +89,7 @@ pub fn not_missing(
             }
         }
         DefinitionKind::Method(stmt) => {
-            if is_overload(stmt) {
+            if is_overload(stmt) || is_override(stmt) {
                 true
             } else if is_magic(stmt) {
                 if checker.settings.enabled.contains(&CheckCode::D105) {
@@ -125,7 +127,7 @@ pub fn one_liner(checker: &mut Checker, definition: &Definition) {
         {
             let mut line_count = 0;
             let mut non_empty_line_count = 0;
-            for line in string.lines() {
+            for line in LinesWithTrailingNewline::from(string) {
                 line_count += 1;
                 if !line.trim().is_empty() {
                     non_empty_line_count += 1;
@@ -135,7 +137,7 @@ pub fn one_liner(checker: &mut Checker, definition: &Definition) {
                 }
             }
 
-            if non_empty_line_count == 1 && line_count > 1 {
+            if non_empty_line_count == 1 && (line_count > 1) {
                 checker.add_check(Check::new(
                     CheckKind::FitsOnOneLine,
                     Range::from_located(docstring),
@@ -386,12 +388,14 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
             ..
         } = &docstring.node
         {
-            let lines: Vec<&str> = string.lines().collect();
+            // Split the docstring into lines.
+            let lines: Vec<&str> = LinesWithTrailingNewline::from(string).collect();
             if lines.len() <= 1 {
                 return;
             }
 
-            let docstring_indent = helpers::indentation(checker, docstring);
+            let docstring_indent = whitespace::indentation(checker, docstring);
+
             let mut has_seen_tab = docstring_indent.contains('\t');
             let mut is_over_indented = true;
             let mut over_indented_lines = vec![];
@@ -408,7 +412,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                     continue;
                 }
 
-                let line_indent = helpers::leading_space(lines[i]);
+                let line_indent = whitespace::leading_space(lines[i]);
 
                 // We only report tab indentation once, so only check if we haven't seen a tab
                 // yet.
@@ -417,7 +421,9 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                 if checker.settings.enabled.contains(&CheckCode::D207) {
                     // We report under-indentation on every line. This isn't great, but enables
                     // autofix.
-                    if !is_blank && line_indent.len() < docstring_indent.len() {
+                    if (i == lines.len() - 1 || !is_blank)
+                        && line_indent.len() < docstring_indent.len()
+                    {
                         let mut check = Check::new(
                             CheckKind::NoUnderIndentation,
                             Range {
@@ -427,7 +433,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                         );
                         if checker.patch(check.kind.code()) {
                             check.amend(Fix::replacement(
-                                helpers::clean(&docstring_indent),
+                                whitespace::clean(&docstring_indent),
                                 Location::new(docstring.location.row() + i, 0),
                                 Location::new(docstring.location.row() + i, line_indent.len()),
                             ));
@@ -464,7 +470,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                 // If every line (except the last) is over-indented...
                 if is_over_indented {
                     for i in over_indented_lines {
-                        let line_indent = helpers::leading_space(lines[i]);
+                        let line_indent = whitespace::leading_space(lines[i]);
                         if line_indent.len() > docstring_indent.len() {
                             // We report over-indentation on every line. This isn't great, but
                             // enables autofix.
@@ -477,7 +483,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                             );
                             if checker.patch(check.kind.code()) {
                                 check.amend(Fix::replacement(
-                                    helpers::clean(&docstring_indent),
+                                    whitespace::clean(&docstring_indent),
                                     Location::new(docstring.location.row() + i, 0),
                                     Location::new(docstring.location.row() + i, line_indent.len()),
                                 ));
@@ -490,7 +496,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                 // If the last line is over-indented...
                 if !lines.is_empty() {
                     let i = lines.len() - 1;
-                    let line_indent = helpers::leading_space(lines[i]);
+                    let line_indent = whitespace::leading_space(lines[i]);
                     if line_indent.len() > docstring_indent.len() {
                         let mut check = Check::new(
                             CheckKind::NoOverIndentation,
@@ -501,7 +507,7 @@ pub fn indent(checker: &mut Checker, definition: &Definition) {
                         );
                         if checker.patch(check.kind.code()) {
                             check.amend(Fix::replacement(
-                                helpers::clean(&docstring_indent),
+                                whitespace::clean(&docstring_indent),
                                 Location::new(docstring.location.row() + i, 0),
                                 Location::new(docstring.location.row() + i, line_indent.len()),
                             ));
@@ -523,7 +529,7 @@ pub fn newline_after_last_paragraph(checker: &mut Checker, definition: &Definiti
         } = &docstring.node
         {
             let mut line_count = 0;
-            for line in string.lines() {
+            for line in LinesWithTrailingNewline::from(string) {
                 if !line.trim().is_empty() {
                     line_count += 1;
                 }
@@ -531,7 +537,7 @@ pub fn newline_after_last_paragraph(checker: &mut Checker, definition: &Definiti
                     let content = checker
                         .locator
                         .slice_source_code_range(&Range::from_located(docstring));
-                    if let Some(last_line) = content.lines().last().map(|line| line.trim()) {
+                    if let Some(last_line) = content.lines().last().map(str::trim) {
                         if last_line != "\"\"\"" && last_line != "'''" {
                             let mut check = Check::new(
                                 CheckKind::NewLineAfterLastParagraph,
@@ -541,7 +547,7 @@ pub fn newline_after_last_paragraph(checker: &mut Checker, definition: &Definiti
                                 // Insert a newline just before the end-quote(s).
                                 let content = format!(
                                     "\n{}",
-                                    helpers::clean(&helpers::indentation(checker, docstring))
+                                    whitespace::clean(&whitespace::indentation(checker, docstring))
                                 );
                                 check.amend(Fix::insertion(
                                     content,
@@ -569,7 +575,7 @@ pub fn no_surrounding_whitespace(checker: &mut Checker, definition: &Definition)
             ..
         } = &docstring.node
         {
-            let mut lines = string.lines();
+            let mut lines = LinesWithTrailingNewline::from(string);
             if let Some(line) = lines.next() {
                 let trimmed = line.trim();
                 if trimmed.is_empty() {
@@ -586,11 +592,11 @@ pub fn no_surrounding_whitespace(checker: &mut Checker, definition: &Definition)
                             .slice_source_code_range(&Range::from_located(docstring))
                             .lines()
                             .next()
-                            .map(|line| line.to_lowercase())
+                            .map(str::to_lowercase)
                         {
-                            for pattern in helpers::TRIPLE_QUOTE_PREFIXES
+                            for pattern in constants::TRIPLE_QUOTE_PREFIXES
                                 .iter()
-                                .chain(helpers::SINGLE_QUOTE_PREFIXES)
+                                .chain(constants::SINGLE_QUOTE_PREFIXES)
                             {
                                 if first_line.starts_with(pattern) {
                                     check.amend(Fix::replacement(
@@ -626,15 +632,15 @@ pub fn multi_line_summary_start(checker: &mut Checker, definition: &Definition)
             ..
         } = &docstring.node
         {
-            if string.lines().nth(1).is_some() {
+            if LinesWithTrailingNewline::from(string).nth(1).is_some() {
                 if let Some(first_line) = checker
                     .locator
                     .slice_source_code_range(&Range::from_located(docstring))
                     .lines()
                     .next()
-                    .map(|line| line.to_lowercase())
+                    .map(str::to_lowercase)
                 {
-                    if helpers::TRIPLE_QUOTE_PREFIXES.contains(&first_line.as_str()) {
+                    if constants::TRIPLE_QUOTE_PREFIXES.contains(&first_line.as_str()) {
                         if checker.settings.enabled.contains(&CheckCode::D212) {
                             checker.add_check(Check::new(
                                 CheckKind::MultiLineSummaryFirstLine,
@@ -668,7 +674,7 @@ pub fn triple_quotes(checker: &mut Checker, definition: &Definition) {
                 .slice_source_code_range(&Range::from_located(docstring))
                 .lines()
                 .next()
-                .map(|line| line.to_lowercase())
+                .map(str::to_lowercase)
             {
                 let starts_with_triple = if string.contains("\"\"\"") {
                     first_line.starts_with("'''")
@@ -870,7 +876,7 @@ pub fn sections(checker: &mut Checker, definition: &Definition) {
             ..
         } = &docstring.node
         {
-            let lines: Vec<&str> = string.lines().collect();
+            let lines: Vec<&str> = LinesWithTrailingNewline::from(string).collect();
             if lines.len() < 2 {
                 return;
             }
@@ -899,7 +905,7 @@ fn blanks_and_section_underline(
 ) {
     let docstring = definition
         .docstring
-        .expect("Sections are only available for docstrings.");
+        .expect("Sections are only available for docstrings");
 
     let mut blank_lines_after_header = 0;
     for line in context.following_lines {
@@ -920,7 +926,7 @@ fn blanks_and_section_underline(
                 // Add a dashed line (of the appropriate length) under the section header.
                 let content = format!(
                     "{}{}\n",
-                    helpers::clean(&helpers::indentation(checker, docstring)),
+                    whitespace::clean(&whitespace::indentation(checker, docstring)),
                     "-".repeat(context.section_name.len())
                 );
                 check.amend(Fix::insertion(
@@ -944,51 +950,7 @@ fn blanks_and_section_underline(
         .chars()
         .all(|char| char.is_whitespace() || char == '-');
 
-    if !dash_line_found {
-        if checker.settings.enabled.contains(&CheckCode::D407) {
-            let mut check = Check::new(
-                CheckKind::DashedUnderlineAfterSection(context.section_name.to_string()),
-                Range::from_located(docstring),
-            );
-            if checker.patch(check.kind.code()) {
-                // Add a dashed line (of the appropriate length) under the section header.
-                let content = format!(
-                    "{}{}\n",
-                    helpers::clean(&helpers::indentation(checker, docstring)),
-                    "-".repeat(context.section_name.len())
-                );
-                check.amend(Fix::insertion(
-                    content,
-                    Location::new(docstring.location.row() + context.original_index + 1, 0),
-                ));
-            }
-            checker.add_check(check);
-        }
-        if blank_lines_after_header > 0 {
-            if checker.settings.enabled.contains(&CheckCode::D412) {
-                let mut check = Check::new(
-                    CheckKind::NoBlankLinesBetweenHeaderAndContent(
-                        context.section_name.to_string(),
-                    ),
-                    Range::from_located(docstring),
-                );
-                if checker.patch(check.kind.code()) {
-                    // Delete any blank lines between the header and content.
-                    check.amend(Fix::deletion(
-                        Location::new(docstring.location.row() + context.original_index + 1, 0),
-                        Location::new(
-                            docstring.location.row()
-                                + context.original_index
-                                + 1
-                                + blank_lines_after_header,
-                            0,
-                        ),
-                    ));
-                }
-                checker.add_check(check);
-            }
-        }
-    } else {
+    if dash_line_found {
         if blank_lines_after_header > 0 {
             if checker.settings.enabled.contains(&CheckCode::D408) {
                 let mut check = Check::new(
@@ -1030,7 +992,7 @@ fn blanks_and_section_underline(
                     // Replace the existing underline with a line of the appropriate length.
                     let content = format!(
                         "{}{}\n",
-                        helpers::clean(&helpers::indentation(checker, docstring)),
+                        whitespace::clean(&whitespace::indentation(checker, docstring)),
                         "-".repeat(context.section_name.len())
                     );
                     check.amend(Fix::replacement(
@@ -1057,8 +1019,8 @@ fn blanks_and_section_underline(
         }
 
         if checker.settings.enabled.contains(&CheckCode::D215) {
-            let leading_space = helpers::leading_space(non_empty_line);
-            let indentation = helpers::indentation(checker, docstring);
+            let leading_space = whitespace::leading_space(non_empty_line);
+            let indentation = whitespace::indentation(checker, docstring);
             if leading_space.len() > indentation.len() {
                 let mut check = Check::new(
                     CheckKind::SectionUnderlineNotOverIndented(context.section_name.to_string()),
@@ -1067,7 +1029,7 @@ fn blanks_and_section_underline(
                 if checker.patch(check.kind.code()) {
                     // Replace the existing indentation with whitespace of the appropriate length.
                     check.amend(Fix::replacement(
-                        helpers::clean(&indentation),
+                        whitespace::clean(&indentation),
                         Location::new(
                             docstring.location.row()
                                 + context.original_index
@@ -1145,6 +1107,50 @@ fn blanks_and_section_underline(
                 ));
             }
         }
+    } else {
+        if checker.settings.enabled.contains(&CheckCode::D407) {
+            let mut check = Check::new(
+                CheckKind::DashedUnderlineAfterSection(context.section_name.to_string()),
+                Range::from_located(docstring),
+            );
+            if checker.patch(check.kind.code()) {
+                // Add a dashed line (of the appropriate length) under the section header.
+                let content = format!(
+                    "{}{}\n",
+                    whitespace::clean(&whitespace::indentation(checker, docstring)),
+                    "-".repeat(context.section_name.len())
+                );
+                check.amend(Fix::insertion(
+                    content,
+                    Location::new(docstring.location.row() + context.original_index + 1, 0),
+                ));
+            }
+            checker.add_check(check);
+        }
+        if blank_lines_after_header > 0 {
+            if checker.settings.enabled.contains(&CheckCode::D412) {
+                let mut check = Check::new(
+                    CheckKind::NoBlankLinesBetweenHeaderAndContent(
+                        context.section_name.to_string(),
+                    ),
+                    Range::from_located(docstring),
+                );
+                if checker.patch(check.kind.code()) {
+                    // Delete any blank lines between the header and content.
+                    check.amend(Fix::deletion(
+                        Location::new(docstring.location.row() + context.original_index + 1, 0),
+                        Location::new(
+                            docstring.location.row()
+                                + context.original_index
+                                + 1
+                                + blank_lines_after_header,
+                            0,
+                        ),
+                    ));
+                }
+                checker.add_check(check);
+            }
+        }
     }
 }
 
@@ -1156,7 +1162,7 @@ fn common_section(
 ) {
     let docstring = definition
         .docstring
-        .expect("Sections are only available for docstrings.");
+        .expect("Sections are only available for docstrings");
 
     if checker.settings.enabled.contains(&CheckCode::D405) {
         if !style
@@ -1189,7 +1195,7 @@ fn common_section(
                                 docstring.location.row() + context.original_index,
                                 section_name_start + section_name_length,
                             ),
-                        ))
+                        ));
                     }
                 }
                 checker.add_check(check);
@@ -1198,8 +1204,8 @@ fn common_section(
     }
 
     if checker.settings.enabled.contains(&CheckCode::D214) {
-        let leading_space = helpers::leading_space(context.line);
-        let indentation = helpers::indentation(checker, docstring);
+        let leading_space = whitespace::leading_space(context.line);
+        let indentation = whitespace::indentation(checker, docstring);
         if leading_space.len() > indentation.len() {
             let mut check = Check::new(
                 CheckKind::SectionNotOverIndented(context.section_name.to_string()),
@@ -1208,7 +1214,7 @@ fn common_section(
             if checker.patch(check.kind.code()) {
                 // Replace the existing indentation with whitespace of the appropriate length.
                 check.amend(Fix::replacement(
-                    helpers::clean(&indentation),
+                    whitespace::clean(&indentation),
                     Location::new(docstring.location.row() + context.original_index, 0),
                     Location::new(
                         docstring.location.row() + context.original_index,
@@ -1223,8 +1229,7 @@ fn common_section(
     if context
         .following_lines
         .last()
-        .map(|line| !line.trim().is_empty())
-        .unwrap_or(true)
+        .map_or(true, |line| !line.trim().is_empty())
     {
         if context.is_last_section {
             if checker.settings.enabled.contains(&CheckCode::D413) {
@@ -1284,18 +1289,14 @@ fn common_section(
                     Location::new(docstring.location.row() + context.original_index, 0),
                 ));
             }
-            checker.add_check(check)
+            checker.add_check(check);
         }
     }
 
     blanks_and_section_underline(checker, definition, context);
 }
 
-fn missing_args(
-    checker: &mut Checker,
-    definition: &Definition,
-    docstrings_args: &FnvHashSet<&str>,
-) {
+fn missing_args(checker: &mut Checker, definition: &Definition, docstrings_args: &FxHashSet<&str>) {
     if let DefinitionKind::Function(parent)
     | DefinitionKind::NestedFunction(parent)
     | DefinitionKind::Method(parent) = definition.kind
@@ -1307,8 +1308,9 @@ fn missing_args(
             args: arguments, ..
         } = &parent.node
         {
-            // Collect all the arguments into a single vector.
-            let mut all_arguments: Vec<&Arg> = arguments
+            // Look for arguments that weren't included in the docstring.
+            let mut missing_arg_names: BTreeSet<String> = BTreeSet::default();
+            for arg in arguments
                 .args
                 .iter()
                 .chain(arguments.posonlyargs.iter())
@@ -1320,33 +1322,38 @@ fn missing_args(
                             && !is_staticmethod(parent),
                     ),
                 )
-                .collect();
+            {
+                let arg_name = arg.node.arg.as_str();
+                if !arg_name.starts_with('_') && !docstrings_args.contains(&arg_name) {
+                    missing_arg_names.insert(arg_name.to_string());
+                }
+            }
+
+            // Check specifically for `vararg` and `kwarg`, which can be prefixed with a
+            // single or double star, respectively.
             if let Some(arg) = &arguments.vararg {
-                all_arguments.push(arg);
+                let arg_name = arg.node.arg.as_str();
+                let starred_arg_name = format!("*{arg_name}");
+                if !arg_name.starts_with('_')
+                    && !docstrings_args.contains(&arg_name)
+                    && !docstrings_args.contains(&starred_arg_name.as_str())
+                {
+                    missing_arg_names.insert(starred_arg_name);
+                }
             }
             if let Some(arg) = &arguments.kwarg {
-                all_arguments.push(arg);
-            }
-
-            // Look for arguments that weren't included in the docstring.
-            let mut missing_args: BTreeSet<&str> = Default::default();
-            for arg in all_arguments {
                 let arg_name = arg.node.arg.as_str();
-                if arg_name.starts_with('_') {
-                    continue;
+                let starred_arg_name = format!("**{arg_name}");
+                if !arg_name.starts_with('_')
+                    && !docstrings_args.contains(&arg_name)
+                    && !docstrings_args.contains(&starred_arg_name.as_str())
+                {
+                    missing_arg_names.insert(starred_arg_name);
                 }
-                if docstrings_args.contains(&arg_name) {
-                    continue;
-                }
-                missing_args.insert(arg_name);
             }
 
-            if !missing_args.is_empty() {
-                let names = missing_args
-                    .into_iter()
-                    .map(String::from)
-                    .sorted()
-                    .collect();
+            if !missing_arg_names.is_empty() {
+                let names = missing_arg_names.into_iter().sorted().collect();
                 checker.add_check(Check::new(
                     CheckKind::DocumentAllArguments(names),
                     Range::from_located(parent),
@@ -1358,7 +1365,7 @@ fn missing_args(
 
 // See: `GOOGLE_ARGS_REGEX` in `pydocstyle/checker.py`.
 static GOOGLE_ARGS_REGEX: Lazy<Regex> =
-    Lazy::new(|| Regex::new(r"^\s*(\w+)\s*(\(.*?\))?\s*:\n?\s*.+").expect("Invalid regex"));
+    Lazy::new(|| Regex::new(r"^\s*(\w+)\s*(\(.*?\))?\s*:\n?\s*.+").unwrap());
 
 fn args_section(checker: &mut Checker, definition: &Definition, context: &SectionContext) {
     let mut args_sections: Vec<String> = vec![];
@@ -1366,12 +1373,7 @@ fn args_section(checker: &mut Checker, definition: &Definition, context: &Sectio
         .trim()
         .lines()
     {
-        if line
-            .chars()
-            .next()
-            .map(|char| char.is_whitespace())
-            .unwrap_or(true)
-        {
+        if line.chars().next().map_or(true, char::is_whitespace) {
             // This is a continuation of documentation for the last
             // parameter because it does start with whitespace.
             if let Some(current) = args_sections.last_mut() {
@@ -1388,25 +1390,28 @@ fn args_section(checker: &mut Checker, definition: &Definition, context: &Sectio
         checker,
         definition,
         // Collect the list of arguments documented in the docstring.
-        &FnvHashSet::from_iter(args_sections.iter().filter_map(|section| {
-            match GOOGLE_ARGS_REGEX.captures(section.as_str()) {
-                Some(caps) => caps.get(1).map(|arg_name| arg_name.as_str()),
-                None => None,
-            }
-        })),
-    )
+        &args_sections
+            .iter()
+            .filter_map(
+                |section| match GOOGLE_ARGS_REGEX.captures(section.as_str()) {
+                    Some(caps) => caps.get(1).map(|arg_name| arg_name.as_str()),
+                    None => None,
+                },
+            )
+            .collect(),
+    );
 }
 
 fn parameters_section(checker: &mut Checker, definition: &Definition, context: &SectionContext) {
     // Collect the list of arguments documented in the docstring.
-    let mut docstring_args: FnvHashSet<&str> = FnvHashSet::default();
-    let section_level_indent = helpers::leading_space(context.line);
+    let mut docstring_args: FxHashSet<&str> = FxHashSet::default();
+    let section_level_indent = whitespace::leading_space(context.line);
     for i in 1..context.following_lines.len() {
         let current_line = context.following_lines[i - 1];
-        let current_leading_space = helpers::leading_space(current_line);
+        let current_leading_space = whitespace::leading_space(current_line);
         let next_line = context.following_lines[i];
         if current_leading_space == section_level_indent
-            && (helpers::leading_space(next_line).len() > current_leading_space.len())
+            && (whitespace::leading_space(next_line).len() > current_leading_space.len())
             && !next_line.trim().is_empty()
         {
             let parameters = if let Some(semi_index) = current_line.find(':') {
@@ -1439,7 +1444,7 @@ fn numpy_section(checker: &mut Checker, definition: &Definition, context: &Secti
         if !suffix.is_empty() {
             let docstring = definition
                 .docstring
-                .expect("Sections are only available for docstrings.");
+                .expect("Sections are only available for docstrings");
             let mut check = Check::new(
                 CheckKind::NewLineAfterSectionName(context.section_name.to_string()),
                 Range::from_located(docstring),
@@ -1464,7 +1469,7 @@ fn numpy_section(checker: &mut Checker, definition: &Definition, context: &Secti
                     ));
                 }
             }
-            checker.add_check(check)
+            checker.add_check(check);
         }
     }
 
@@ -1488,7 +1493,7 @@ fn google_section(checker: &mut Checker, definition: &Definition, context: &Sect
         if suffix != ":" {
             let docstring = definition
                 .docstring
-                .expect("Sections are only available for docstrings.");
+                .expect("Sections are only available for docstrings");
             let mut check = Check::new(
                 CheckKind::SectionNameEndsInColon(context.section_name.to_string()),
                 Range::from_located(docstring),
