@@ -1,5 +1,8 @@
 use anyhow::Result;
-use libcst_native::{Codegen, Expression, ImportNames, SmallStatement, Statement};
+use libcst_native::{
+    Codegen, CodegenState, Expression, ImportNames, ParenthesizableWhitespace, SmallStatement,
+    Statement,
+};
 use rustpython_ast::{Expr, Keyword, Location, Stmt};
 use rustpython_parser::lexer;
 use rustpython_parser::lexer::Tok;
@@ -10,10 +13,10 @@ use crate::autofix::{self, Fix};
 use crate::cst::matchers::match_module;
 use crate::source_code_locator::SourceCodeLocator;
 
-/// Generate a fix to remove a base from a ClassDef statement.
+/// Generate a fix to remove a base from a `ClassDef` statement.
 pub fn remove_class_def_base(
     locator: &SourceCodeLocator,
-    stmt_at: &Location,
+    stmt_at: Location,
     expr_at: Location,
     bases: &[Expr],
     keywords: &[Keyword],
@@ -28,7 +31,7 @@ pub fn remove_class_def_base(
         for (start, tok, end) in lexer::make_tokenizer(&contents).flatten() {
             if matches!(tok, Tok::Lpar) {
                 if count == 0 {
-                    fix_start = Some(helpers::to_absolute(&start, stmt_at));
+                    fix_start = Some(helpers::to_absolute(start, stmt_at));
                 }
                 count += 1;
             }
@@ -36,7 +39,7 @@ pub fn remove_class_def_base(
             if matches!(tok, Tok::Rpar) {
                 count -= 1;
                 if count == 0 {
-                    fix_end = Some(helpers::to_absolute(&end, stmt_at));
+                    fix_end = Some(helpers::to_absolute(end, stmt_at));
                     break;
                 }
             }
@@ -59,7 +62,7 @@ pub fn remove_class_def_base(
         let mut fix_end: Option<Location> = None;
         let mut seen_comma = false;
         for (start, tok, end) in lexer::make_tokenizer(&contents).flatten() {
-            let start = helpers::to_absolute(&start, stmt_at);
+            let start = helpers::to_absolute(start, stmt_at);
             if seen_comma {
                 if matches!(tok, Tok::Newline) {
                     fix_end = Some(end);
@@ -77,7 +80,7 @@ pub fn remove_class_def_base(
         }
 
         match (fix_start, fix_end) {
-            (Some(start), Some(end)) => Some(Fix::replacement("".to_string(), start, end)),
+            (Some(start), Some(end)) => Some(Fix::replacement(String::new(), start, end)),
             _ => None,
         }
     } else {
@@ -86,8 +89,8 @@ pub fn remove_class_def_base(
         let mut fix_start: Option<Location> = None;
         let mut fix_end: Option<Location> = None;
         for (start, tok, end) in lexer::make_tokenizer(&contents).flatten() {
-            let start = helpers::to_absolute(&start, stmt_at);
-            let end = helpers::to_absolute(&end, stmt_at);
+            let start = helpers::to_absolute(start, stmt_at);
+            let end = helpers::to_absolute(end, stmt_at);
             if start == expr_at {
                 fix_end = Some(end);
                 break;
@@ -98,7 +101,7 @@ pub fn remove_class_def_base(
         }
 
         match (fix_start, fix_end) {
-            (Some(start), Some(end)) => Some(Fix::replacement("".to_string(), start, end)),
+            (Some(start), Some(end)) => Some(Fix::replacement(String::new(), start, end)),
             _ => None,
         }
     }
@@ -108,19 +111,16 @@ pub fn remove_super_arguments(locator: &SourceCodeLocator, expr: &Expr) -> Optio
     let range = Range::from_located(expr);
     let contents = locator.slice_source_code_range(&range);
 
-    let mut tree = match libcst_native::parse_module(&contents, None) {
-        Ok(m) => m,
-        Err(_) => return None,
-    };
+    let mut tree = libcst_native::parse_module(&contents, None).ok()?;
 
     if let Some(Statement::Simple(body)) = tree.body.first_mut() {
         if let Some(SmallStatement::Expr(body)) = body.body.first_mut() {
             if let Expression::Call(body) = &mut body.value {
                 body.args = vec![];
-                body.whitespace_before_args = Default::default();
-                body.whitespace_after_func = Default::default();
+                body.whitespace_before_args = ParenthesizableWhitespace::default();
+                body.whitespace_after_func = ParenthesizableWhitespace::default();
 
-                let mut state = Default::default();
+                let mut state = CodegenState::default();
                 tree.codegen(&mut state);
 
                 return Some(Fix::replacement(
@@ -147,23 +147,17 @@ pub fn remove_unnecessary_future_import(
     let module_text = locator.slice_source_code_range(&Range::from_located(stmt));
     let mut tree = match_module(&module_text)?;
 
-    let body = if let Some(Statement::Simple(body)) = tree.body.first_mut() {
-        body
-    } else {
-        return Err(anyhow::anyhow!("Expected node to be: Statement::Simple"));
+    let Some(Statement::Simple(body)) = tree.body.first_mut() else {
+        return Err(anyhow::anyhow!("Expected Statement::Simple"));
     };
-    let body = if let Some(SmallStatement::ImportFrom(body)) = body.body.first_mut() {
-        body
-    } else {
+    let Some(SmallStatement::ImportFrom(body)) = body.body.first_mut() else {
         return Err(anyhow::anyhow!(
-            "Expected node to be: SmallStatement::ImportFrom"
+            "Expected SmallStatement::ImportFrom"
         ));
     };
 
-    let aliases = if let ImportNames::Aliases(aliases) = &mut body.names {
-        aliases
-    } else {
-        return Err(anyhow::anyhow!("Expected node to be: Aliases"));
+    let ImportNames::Aliases(aliases) = &mut body.names else {
+        return Err(anyhow::anyhow!("Expected Aliases"));
     };
 
     // Preserve the trailing comma (or not) from the last entry.
@@ -181,7 +175,7 @@ pub fn remove_unnecessary_future_import(
     if aliases.is_empty() {
         autofix::helpers::remove_stmt(stmt, parent, deleted)
     } else {
-        let mut state = Default::default();
+        let mut state = CodegenState::default();
         tree.codegen(&mut state);
 
         Ok(Fix::replacement(
@@ -191,34 +185,3 @@ pub fn remove_unnecessary_future_import(
         ))
     }
 }
-
-/// U011
-pub fn remove_unnecessary_lru_cache_params(
-    locator: &SourceCodeLocator,
-    decor_at: &Location,
-) -> Option<Fix> {
-    let contents = locator.slice_source_code_at(decor_at);
-    let mut fix_start = None;
-    let mut fix_end = None;
-    let mut count: usize = 0;
-    for (start, tok, end) in lexer::make_tokenizer(&contents).flatten() {
-        if matches!(tok, Tok::Lpar) {
-            if count == 0 {
-                fix_start = Some(helpers::to_absolute(&start, decor_at));
-            }
-            count += 1;
-        }
-
-        if matches!(tok, Tok::Rpar) {
-            count -= 1;
-            if count == 0 {
-                fix_end = Some(helpers::to_absolute(&end, decor_at));
-                break;
-            }
-        }
-    }
-    match (fix_start, fix_end) {
-        (Some(start), Some(end)) => Some(Fix::deletion(start, end)),
-        _ => None,
-    }
-}
