@@ -22,7 +22,7 @@ pub fn super_call_with_parameters(checker: &mut Checker, expr: &Expr, func: &Exp
                     check.amend(fix);
                 }
             }
-            checker.add_check(check)
+            checker.add_check(check);
         }
     }
 }
