@@ -25,10 +25,10 @@ pub fn useless_metaclass_type(checker: &mut Checker, stmt: &Stmt, value: &Expr,
                 &deleted,
             ) {
                 Ok(fix) => {
-                    if fix.patch.content.is_empty() || fix.patch.content == "pass" {
+                    if fix.content.is_empty() || fix.content == "pass" {
                         checker.deletions.insert(context.defined_by);
                     }
-                    check.amend(fix)
+                    check.amend(fix);
                 }
                 Err(e) => error!("Failed to fix remove metaclass type: {}", e),
             }
